@@ -1,4 +1,4 @@
-from megadata.mypy import *
+from .mypy import *
 
 def on_quit(*a):
   print('on_quit',*a)
@@ -23,7 +23,9 @@ def clt_ipc(argv):
     if line.startswith('lambda'): # TMP TEST RPC
       fc = eval(line)
       #print('debug fc',fc,fc.__code__)
-      print(server(dumps_func(fc)))
+      b = dumps_func(fc)
+      #print('debug fc',len(b),b)
+      print(server(b))
 
     # e.g. /"Adm","ping"
     # have fund like /"Adm","ping",*args,**kwargs
@@ -49,5 +51,3 @@ def clt_ipc(argv):
       rt = tryx(lambda:server(line))
       print(type(rt),'=>',rt)
 
-if __name__ == '__main__':
-  clt_ipc(argv)
