@@ -1,17 +1,11 @@
 #-*- coding: utf-8 -*-
 
-# py-coding-simplifer by Wanjo 20230323
-# 20230323: dropped _print, and fix try_asyncio
-
-#from __future__ import print_function # for py2 print()
-#_print=print
+# py-coding-simplifer by Wanjo
 
 from time import time as now, mktime, sleep
 
-# for audit
 load_time = now()
 
-# core of mypy
 #exec('def tryx(l,e=print):\n try:return l()\n except Exception as ex:return ex if True==e else e(ex) if e else None')
 def tryx(l,e=print):
     try: return l()
@@ -36,6 +30,7 @@ touch_dir = lambda fn:os.makedirs(fn,exist_ok=True)
 file_exists = os.path.exists
 get_mtime = lambda f:os.path.getmtime(f)
 
+# rename or delete
 def file_rename(fnold,fnnew=None,try_delete_after=False):
     if try_delete_after:
         fnnew = f'{fnnew}.delete.{now()}'
@@ -43,11 +38,12 @@ def file_rename(fnold,fnnew=None,try_delete_after=False):
     if try_delete_after:
         tryx(lambda:os.remove(fnnew))
 
-evalx = lambda s,g=globals():eval(s,g)
+evalx = lambda s,g=globals(),l=locals():eval(s,g,l)
 
 flag_py2 = sys.version_info.major==2
-sys_import = __import__
-sys_reload = __builtins__.reload if flag_py2 else sys_import('importlib').reload
+import importlib
+sys_import = importlib.import_module
+sys_reload = importlib.reload
 refresh = lambda n:sys_reload(sys_import(n))
 
 class probe:
@@ -63,10 +59,30 @@ class probex:
       if rt is None: rt = tryx(lambda:sys_import(k),self.debug)
       return rt
 
-# from megadata.mypy import mypy
 mypy = probex(evalx)
 
-if not flag_py2:
+import marshal,types
+
+def use(mdlname,clsname=None,reload=False):
+  rt = None
+  if type(mdlname) is str:
+    if reload:
+      rt = refresh(mdlname)
+    else:
+      rt = sys_import(mdlname)
+  else:
+    rt = mdlname
+  if type(clsname) is str:
+    #return tryx(lambda:getattr(rt,clsname))
+    for v in clsname.split('.'):
+      rt = tryx(lambda:getattr(rt,v))
+      if rt is None: break # safety
+  return rt
+
+useapi = lambda c,m,*args,**kwargs: getattr(use(c,'api')(*args,**kwargs),m)
+
+#if not flag_py2: # patch for some urlopen case
+if True:
     # https://stackoverflow.com/questions/18466079/change-the-connection-pool-size-for-pythons-requests-module-when-in-threading/22253656#22253656
     def patch_connection_pool(**constructor_kwargs):
         from urllib3 import connectionpool, poolmanager
@@ -91,20 +107,21 @@ import json
 class MyJsonEncoder(json.JSONEncoder):
     def default(self, obj): return tryx(lambda:json.JSONEncoder.default(self,obj))
 s2o = lambda s:tryx(lambda:json.loads(s),False)
-o2s = lambda o,indent=None:tryx(lambda:json.dumps(o, indent=indent, ensure_ascii=False, cls=MyJsonEncoder))
+o2s = lambda o,indent=None,separators=(',',':'):tryx(lambda:json.dumps(o, indent=indent, ensure_ascii=False, cls=MyJsonEncoder, separators=separators))
 
 from pickle import dumps as o2b, loads as b2o
 
+from urllib.request import urlopen
 def get_urlopen():
-  if flag_py2: from urllib2 import urlopen
-  else: from urllib.request import urlopen
+  #if flag_py2: from urllib2 import urlopen
+  #else: from urllib.request import urlopen
   return urlopen
 
-# NOTES: one-off simple web call, not for heavy usage!!
+# NOTES: one-off simple web call, not for heavy usage!! using aiohttp instead!
+
 wc=lambda u=None, data=None, m='POST', timeout=10:get_urlopen()(url=u,data=data.encode('utf-8') if isinstance(data,str) else o2s(data).encode('utf-8') if data else None,timeout=timeout).read().decode()
 
-# NOTES: one-off ws call, not for heavy usage!!!
-# TODO: need to fix for non-blocking...
+# NOTES: one-off ws call, not for heavy usage!!! has problem of IO-blocking
 def wsc(u,data,lines=1):
   s=data.encode('utf-8') if isinstance(data,str) else o2s(data).encode('utf-8') if data else None
 
@@ -163,6 +180,16 @@ def ipc(u,data,authkey=None,out=None,timeout=7):
     return rt_loads
   return rt
 
+build_api_closure=lambda *args,**kwargs:eval(f'lambda:api(*{args},**{kwargs})')
+
+def rpc(u,authkey=None,out=None,timeout=7):
+  def rpc_func(*rpc_args,**rpc_kwargs):
+    c = build_api_closure(*rpc_args,**rpc_kwargs)
+    b = dumps_func(c)
+    s=ipc(u,b,authkey=authkey,out=out,timeout=timeout)
+    return s2o(s)
+  return rpc_func
+
 read = lambda f,m='r',encoding='utf-8':open(f,m,encoding=encoding).read()
 # for binary: write(f,s,'wb',None)
 write = lambda f,s,m='w',encoding='utf-8':open(f,m,encoding=encoding).write(s)
@@ -171,7 +198,6 @@ save = lambda f,o:write(f,o2s(o))
 
 log = lambda fn,o,ln='\n':write(fn, f'{o if type(o) is str else o2s(o)}{ln}', 'a')
 
-import marshal,types
 dumps_func = lambda func:marshal.dumps(func.__code__)
 loads_func = lambda codes,ctx,name=None:types.FunctionType(marshal.loads(codes),ctx,name=name)
 func2file = lambda fc,fn:write(fn,dumps_func(fc),'wb',None)
@@ -192,7 +218,7 @@ class objx(dict):#dictxx
     def __setattr__(self,k,v): self[k]=v
 
 def on_quit_default(*a):
-  print(*a)
+  print('on_quit_default',*a)
   os._exit(0)
 
 def hook_quit(on_quit=on_quit_default):
@@ -297,21 +323,10 @@ def yielder_loop(func,wrap=tryx,do_yield=True):
     rt = yield from yielder(func, wrap, do_yield)
     if do_yield and rt is not None: yield rt
 
-#import threading
-#Thread = threading.Thread
 Thread = mypy.threading.Thread
-#try_async=lambda func:tryx(lambda:threading.Thread(target=func).start())
-#try_async=lambda func:Thread(target=func).start()
 def try_async(func): Thread(target=func).start()
 
-import asyncio
-from asyncio import new_event_loop
-#def try_asyncio(func): return new_event_loop().run_in_executor(None,func)
-#try_asyncio = lambda func,*args:new_event_loop().run_in_executor(None,func,*args)
-try_asyncio = lambda func:new_event_loop().run_in_executor(None,func)
-#async def try_asyncio(func): return await new_event_loop().run_in_executor(None,func)
-  
-# for ipc
+# for ipc()
 def build_address(arg1,arg2=None,folder='../tmp/'):
   port = tryx(lambda:int(arg1),False)
   if port is None:
@@ -330,7 +345,6 @@ import subprocess
 #  _stdout = sys.stdout if stdout is None else stdout
 #  with subprocess.Popen(cmd, stdout=_stdout, stdin=subprocess.PIPE,creationflags=creationflags) as p:
 #      return p.communicate(input=w.encode() if type(w) is str else w)[0] if w else p.stdout.read() if p.stdout is not None and stdout is not None else None
-#
 CREATE_NO_WINDOW = 0x08000000
 def systemx(cmd,w=None,stdout=subprocess.PIPE,stdin=subprocess.PIPE,stderr=subprocess.PIPE,creationflags=0):
   _stdout = sys.stdout if stdout is None else stdout
@@ -346,8 +360,61 @@ def system(cmd_or_a,stdout_only=True,audit=None):
   if audit: audit(rt)
   return rt
 
-def use(mdlname,clsname=None):
-  rt = sys_import(mdlname)
-  if clsname: return tryx(lambda:getattr(rt,clsname))
-  return rt
+#################### async tools
 
+def rpcx(u,authkey=None,out=None,timeout=7):
+  async def rpc_func(*rpc_args,**rpc_kwargs):
+    c = build_api_closure(*rpc_args,**rpc_kwargs)
+    b = dumps_func(c)
+    s=await ipcx(u,b,authkey=authkey,out=out,timeout=timeout)
+    return s2o(s)
+  return rpc_func
+
+import asyncio
+from asyncio import iscoroutinefunction,iscoroutine,run as asyncio_run,get_event_loop,new_event_loop,set_event_loop,sleep as sleep_async
+
+from asyncio import Semaphore,gather,wait_for
+
+# asyncio version of parallel()
+async def parallelx(async_func, a, pool_size=None, timeout=30):
+    async def limited_concurrent_tasks(semaphore, async_func, arg):
+        async with semaphore:
+            #return await async_func(arg)
+            return await wait_for(async_func(arg),timeout=timeout)
+    if pool_size is None:
+      from os import cpu_count
+      pool_size =  cpu_count()
+    semaphore = Semaphore(pool_size)
+    from concurrent.futures import ThreadPoolExecutor
+    with ThreadPoolExecutor() as pool:
+        loop = get_event_loop()
+        #tasks = [loop.create_task(async_func(arg)) for arg in a]
+        tasks = [loop.create_task(limited_concurrent_tasks(semaphore,async_func,arg)) for arg in a]
+        return await gather(*tasks)
+
+from inspect import isawaitable as is_awaitable
+#is_awaitable = lambda obj: iscoroutinefunction(obj) or iscoroutine(obj)
+
+async def try_await(o): return await o if is_awaitable(o) else o
+
+try_asyncio = lambda sync_func,new=False,executor=None:(new_event_loop if new else get_event_loop)().run_in_executor(executor,sync_func)
+async def try_asyncio_async(sync_func,new=False): return await try_asyncio(sync_func,new)
+
+def run_until_complete(fn,new=True,timeout=0):
+  if callable(fn): new=False
+  loop = (new_event_loop if new else get_event_loop)()
+  async_o = try_asyncio_async(fn) if callable(fn) else try_await(fn) if timeout==0 else wait_for(try_await(fn),timeout=timeout)
+  return loop.run_until_complete(async_o)
+
+#diff from js, it's not a Promise...
+#XXX: ipcx = lambda *args,**kwargs:try_asyncio(lambda:ipc(*args,**kwargs))
+async def ipcx(*args,**kwargs):
+    return await try_asyncio(lambda:ipc(*args,**kwargs))
+
+#from asyncio import sleep as sleep_async,new_event_loop
+#new_sleep = lambda t:new_event_loop().run_until_complete(sleep_async(t))
+
+#################### DELETED
+#sys_reload = __builtins__.reload if flag_py2 else sys_import('importlib').reload
+#sys_import = __import__
+#flag_py2 = sys.version_info.major==2
