@@ -1,9 +1,9 @@
 #-*- coding: utf-8 -*-
 
 if __name__ == '__main__':
-  from Application import *
+  from .svr_ipc_bin import my_main_ipc,on_quit,start_stdin
+  from .mypy import hook_quit,build_address,try_async,try_asyncio,argc,argv
 
-  from svr_ipc_bin import my_main_ipc,on_quit,start_stdin
   hook_quit(on_quit)
 
   async_call = try_asyncio # 5.225,4.4727
